%Script for calculating the electrical current of the material using the
%equilibrium values of hydrogen adsorption free energy and atomic positions
%Joakim Ekspong 01-09-2020

clear all
folder='/Volumes/HDD/GoogleDrive/Jobb/MicroKinetic_manuscript/FinalModel_script/outputfilesSIESTA/'; %pathway to datafiles
position=[100 400 290*1.2 290];
set(0, 'DefaultAxesFontSize',14,'DefaultLineLinewidth',1.0,'DefaultaxesLinewidth',1,'Defaultaxesfontweight', 'normal', 'DefaultFigurePosition', position);

%Define the area where the H-atoms are calculated from (periodic calculations) Otherwise, set to 0
xcellmat=11.0135;
ycellmat=9.5302;
%xcellmat=0;
%ycellmat=0;

filename=[folder 'final_xyz_Hsites.dat']; %[nr x y z species]
fid = fopen(filename,'r');
xyz = textscan(fid,'%f%f%f%f%f');
fclose(fid);
xmat=xyz{2};
ymat=xyz{3};
zmat=xyz{4};

filename=[folder 'finalrunGE.dat']; %[nr GE]
fid = fopen(filename,'r');
finalGE = textscan(fid,'%f%f');
fclose(fid);
GEmat=finalGE{2};

if xcellmat~=0
    matdens=length(GEmat)/(xcellmat*10^-8*ycellmat*10^-8); %sites/cm2
else
    matdens=1.4476e+15; %manually define the density of sites here for slabs etc.
end

perc=1; %1=normal, equal H-sites as nr of atoms. Can be alternated for increasing/decreasing H-site density
GEmatini=GEmat;
GEcov='true';
depmat=0.15; %coverage dependency (eV/ML)
Gcovdist=15; %radius from atom that the GE should be dependent on the total H-coverage

%parameters determining the exchange rates
GR1=10^(-0.9);
GR2=10^(-2.5);
GR3=10^(-1.9);

Ptdens=1.4476e+15; %For Pt-fitted volcano plot
GR1=GR1/(Ptdens);
GR2=GR2/(Ptdens);
GR3=GR3/(Ptdens);

%Include Tafel equation or use only discharge -> atom+ion reactions
reactions='all';
nbr=3; %how many Ang between neighbooring sites where the Tafel reaction is able to occur
%reactions='noTafel';
if strcmp(reactions,'noTafel')==1
    GR3=0;
end

jd=100; %figure numbers
opstep=0.0005; %op=overpotential
opsteps=500;

%General parameters
el=1;
k=0.00008617333262145;
T=298.15;
RTF=k*T;
alpha=0.5; %H adsorption, Volmer symmetry factor
beta=0.5; %atom+ion desorption, Heyrovsky symmetry factor
gamma=0.5; %combination reaction, Tafel symmetry factor
prot=1; %proton activity
press=1; %H2 pressure




%% General volcano plot part (can skip) slow when using vpa
G11=(-1.8:0.1:-0.2);
G11=[G11 (-0.19:0.02:0.19)];
G11=[G11 (0.2:0.1:1.8)];
OP=0;
VP=[0.01,0.05,0.1,0.15,0.2,0.3,0.5]; %potentials where the volcano plot is shown
accuracy='low'; %Set to 'high' for using vpa
for pp=1:opsteps
    OPP(pp)=OP;
    if mod(pp,opsteps/10)==0
        disp([mat2str(pp/opsteps*100), ' %'])
    end
    
    KL2=exp(G11./(k*T));
    if pp==1
       covpot=1./(1+KL2); 
    end
    equi11=KL2;
    equi22=KL2.^-1;
    %FORWARDS AND BACKWARD reactions for coverage calculation-----
    K1=GR1*prot.*equi11.^(-alpha).*exp(-alpha*OP/(k*T));
    K1n=GR1.*equi11.^(1-alpha).*exp((1-alpha)*OP/(k*T));
    K2=GR2.*prot.*equi22.^(-beta).*exp(-1*beta*OP/(k*T));
    K2n=GR2*press.*equi22.^(1-beta).*exp((1-beta)*OP/(k*T));
    
    K3=covpot.*GR3.*exp(2.*gamma.*G11./(k*T));
    K3n=(1-covpot).*GR3*press.*exp(-2.*(1-gamma).*G11./(k*T));
    
    if strcmp(accuracy,'high')==1
        OLD = digits(35);
        covpot=(vpa(K1)+vpa(K2n)+vpa(K3n))./(vpa(K1)+vpa(K1n)+vpa(K2)+vpa(K2n)+vpa(K3)+vpa(K3n));
        digits(OLD);
    else
        covpot=(K1+K2n+K3n)./(K1+K1n+K2+K2n+K3+K3n);
    end
    covpotminus=abs(1-covpot);%numerical errors at low GE, since covpot is equal 1 and i011f=0
    
    %Forward - backward reaction rates
    i011f=el.*Ptdens.*GR1*prot.*covpotminus.*equi11.^(-alpha).*exp(-alpha*OP/(k*T));
    i011b=el.*Ptdens.*GR1.*covpot.*equi11.^(1-alpha).*exp((1-alpha)*OP/(k*T));
    
    i021f=el.*Ptdens.*GR2.*prot.*covpot.*equi22.^(-beta).*exp(-1*beta*OP/(k*T));
    i021b=el.*Ptdens.*GR2*press.*covpotminus.*equi22.^(1-beta).*exp((1-beta)*OP/(k*T));
    
    i031f=el.*Ptdens.*GR3.*covpot.^2.*exp(2.*gamma.*G11./(k*T)); %assume equal sites
    i031b=el.*Ptdens.*GR3*press.*covpotminus.^2.*exp(-2.*(1-gamma).*G11./(k*T)); %assume equal sites
    if pp==1
        i011s=i011f;
    end
    if pp==1
        i021s=i021f;
    end
    if pp==1
        i031s=i031f;
    end
    
    i011=(i011f-i011b);
    i021=(i021f-i021b);
    i031=(i031f-i031b);
    
    i0full=i011+i021;
        
    %Plot Volcano plot
    if any(abs(OP+VP)<0.001)==1
        figure(jd*1)
        h11=plot(G11,log10((i011)),'--g','MarkerSize',1);
        hold on
        h21=plot(G11,log10((i021)),'--b','MarkerSize',1);
        hold on
        h31=plot(G11,log10((i031)),'--r','MarkerSize',1);
        hold on
        h41=plot(G11,log10(abs(i0full)),'k','LineWidth',2);
        hold on
        ylim([-4.1,3.2])
        xlim([-0.41,0.41])
        xticks([-0.4,-0.2,0,0.2,0.4])
        xlabel('\DeltaG_{H} (eV)')
        ylabel('log_{10}(J) (A/cm^{2})')
    end
    OP=OP-opstep;
    
end %Overpotential loop

i0fulls=2.*i021s+i031s;
figure(jd*2)
plot(G11,log10(i0fulls),'k','LineWidth',2);
hold on
plot(G11,log10(i011s),'--g','LineWidth',1);
hold on
plot(G11,log10(i021s),'--b','LineWidth',1);
hold on
plot(G11,log10(i031s),'--r','LineWidth',1);
ylim([-8.6,0])
xlim([-0.90,0.90])
xticks([-0.8,-0.6,-0.4,-0.2,0,0.2,0.4,0.6,0.8])
xlabel('\DeltaG_{H} (eV)')
ylabel('log_{10}(J_{0}) (A/cm^{2})')
hold on
grid on

%% User defined structure and energies
OP=0;
for pp=1:opsteps
    OPP(pp)=OP;
    if mod(pp,opsteps/10)==0
        disp([mat2str(pp/opsteps*100), ' %'])
    end
    %Coverage dependency
    if strcmp(GEcov,'true')==1
        if pp>1
            for y=1:length(GEmat)
                arG=sqrt(abs(xmat(y)-xmat(:)).^2+abs(ymat(y)-ymat(:)).^2+abs(zmat(y)-zmat(:)).^2);
                [adisG, bdisG]=find(arG<Gcovdist);
                TotcovGini(y)=sum(cell2mat(covpotnewPtini(1,adisG)))/length(adisG);
                TotcovG(y)=sum(cell2mat(covpotnewmat(pp-1,adisG)))/length(adisG);
                GEmat(y)=GEmatini(y)+(TotcovG(y)-TotcovGini(y)).*depmat;
            end
        end
    end
    
    KL2Pt=exp(GEmat./(k*T));
    equi1Pt=prot.*KL2Pt;
    equi2Pt=prot./press.*KL2Pt.^-1;
   
    for oo=1:length(GEmat)
        if pp==1
            covpotnewmat{pp,oo}=1/(1+KL2Pt(oo)^(1));
            covpotnewPtini{pp,oo}=1/(1+KL2Pt(oo)^(1));
        else
            covpotnewmat{pp,oo}=covpotnewmat{pp-1,oo};
        end
    end
    for oo=1:length(GEmat)
        
        K1Pt=GR1*prot.*equi1Pt(oo).^(-alpha).*exp(-alpha*OP/(k*T));
        K1nPt=(GR1.*equi1Pt(oo).^(1-alpha).*exp((1-alpha)*OP/(k*T)));
        K2Pt=(GR2.*prot.*equi2Pt(oo).^(-beta).*exp(-1*beta*OP/(k*T)));
        K2nPt=(GR2*press.*equi2Pt(oo).^(1-beta).*exp((1-beta)*OP/(k*T)));
        
        %Here the neighbooring sites to site (oo) are found for the
        %combination reaciton including neighboring unit cells
        xxPt=xmat(oo);
        yyPt=ymat(oo);
        zzPt=zmat(oo);
        ar=sqrt(abs(xmat(:)-xxPt).^2+abs(ymat(:)-yyPt).^2+abs(zmat(:)-zzPt).^2);
        [adis, bdis]=find(ar<nbr);
        adis=adis';
        if xcellmat ~=0
            axp=sqrt(abs(xmat(:)-xxPt+xcellmat).^2+abs(ymat(:)-yyPt).^2+abs(zmat(:)-zzPt).^2);
            axn=sqrt(abs(xmat(:)-xxPt-xcellmat).^2+abs(ymat(:)-yyPt).^2+abs(zmat(:)-zzPt).^2);
            ayp=sqrt(abs(xmat(:)-xxPt).^2+abs(ymat(:)-yyPt+ycellmat).^2+abs(zmat(:)-zzPt).^2);
            ayn=sqrt(abs(xmat(:)-xxPt).^2+abs(ymat(:)-yyPt-ycellmat).^2+abs(zmat(:)-zzPt).^2);
            
            [adisxp, bdisxp]=find(axp<nbr);
            [adisxn, bdisxn]=find(axn<nbr);
            [adisyp, bdisyp]=find(ayp<nbr);
            [adisyn, bdisyn]=find(ayn<nbr);
            
            adis=[adis adisxp' adisxn' adisyp' adisyn'];
        end
        adis(find(adis==oo))=[]; %remove self interaction
        adisf{oo}=adis; %atoms that oo interacts with
        
        B=GEmat(adisf{oo});
        C=cell2mat(covpotnewmat(pp,adisf{oo}));
        
        K3Pt=GR3.*exp(1.*gamma.*GEmat(oo)./(k*T)).*max(C(:).*exp(1.*gamma.*B(:)./(k*T)));
        K3nPt=GR3.*press.*exp(-1.*(1-gamma).*GEmat(oo)./(k*T)).*max((1-C(:)).*exp(-1.*(1-gamma).*B(:)./(k*T)));
        
        covpotmat(oo)=(K1Pt+K2nPt+K3nPt)./(K1Pt+K2nPt+K1nPt+K2Pt+K3nPt+K3Pt);
        covpotmat2(oo)=double(covpotmat(oo));
        covpotnew2mat{pp,oo}=covpotmat2(oo);
    end
    covpotoldmat=covpotnewmat;
    covpotnewmat=covpotnew2mat;
    
    covpotmattot{pp}=sum(cell2mat(covpotnewmat(pp,:)))/length(GEmat).*perc;
    
    for oo=1:length(GEmat)
        
        i011fPt{pp,oo}=el.*(GR1*prot.*(1-covpotnewmat{pp,oo}).*equi1Pt(oo).^(-alpha).*exp(-alpha*OP/(k*T)));
        i011bPt{pp,oo}=el.*(GR1.*covpotnewmat{pp,oo}.*equi1Pt(oo).^(1-alpha).*exp((1-alpha)*OP/(k*T)));
        
        i021fPt{pp,oo}=el.*(GR2.*prot.*equi2Pt(oo).^(-beta).*covpotnewmat{pp,oo}.*exp(-1*beta*OP/(k*T)));
        i021bPt{pp,oo}=el.*(GR2*press.*equi2Pt(oo).^(1-beta).*(1-covpotnewmat{pp,oo}).*exp((1-beta)*OP/(k*T)));
        
        B=GEmat(adisf{oo});
        if pp==1
            C=cell2mat(covpotnewmat(pp,adisf{oo}));
        else
            C=cell2mat(covpotnewmat(pp,adisf{oo}));
        end
        i031fPt{pp,oo}=el.*GR3.*covpotnewmat{pp,oo}.*exp(gamma.*GEmat(oo)./(k*T)).*max(C(:).*exp(gamma.*B(:)./(k*T)));
        i031bPt{pp,oo}=el.*GR3*press.*(1-covpotnewmat{pp,oo}).*exp(-1.*(1-gamma).*GEmat(oo)./(k*T)).*max((1-C(:)).*exp(-1.*(1-gamma).*B(:)./(k*T)));
        
        i011Pt{pp,oo}=(i011fPt{pp,oo}-i011bPt{pp,oo});
        i021Pt{pp,oo}=(i021fPt{pp,oo}-i021bPt{pp,oo});
        i031Pt{pp,oo}=(i031fPt{pp,oo}-i031bPt{pp,oo});
        
        IIPt{pp,oo}=-i011Pt{pp,oo}-i021Pt{pp,oo};
        
    end
    
    %Here the individual rates are saved
    IImatfull{pp}=matdens.*sum(cell2mat(IIPt(pp,:)))/length(GEmat).*perc;
    
    IImatf1f{pp}=matdens.*sum(cell2mat(i011fPt(pp,:)))/length(GEmat).*perc;
    IImatf1b{pp}=matdens.*sum(cell2mat(i011bPt(pp,:)))/length(GEmat).*perc;
    IImatf2f{pp}=matdens.*sum(cell2mat(i021fPt(pp,:)))/length(GEmat).*perc;
    IImatf2b{pp}=matdens.*sum(cell2mat(i021bPt(pp,:)))/length(GEmat).*perc;
    IImatf3f{pp}=matdens.*sum(cell2mat(i031fPt(pp,:)))/length(GEmat).*perc;
    IImatf3b{pp}=matdens.*sum(cell2mat(i031bPt(pp,:)))/length(GEmat).*perc;
    IImatf1t{pp}=matdens.*sum(cell2mat(i011Pt(pp,:)))/length(GEmat).*perc;
    IImatf2t{pp}=matdens.*sum(cell2mat(i021Pt(pp,:)))/length(GEmat).*perc;
    IImatf3t{pp}=matdens.*sum(cell2mat(i031Pt(pp,:)))/length(GEmat).*perc;
    
    %This is the exchange current density
    if pp==1
        i0fullsmat=2.*cell2mat(IImatf2f(1))+cell2mat(IImatf3f(1));
    end
    
    %to avoid instability close to GE=0 eV
    if pp>1
        covpotmattot{pp}=(covpotmattot{pp}+covpotmattot{pp-1})./2;
        for yy=1:length(GEmat)
            covpotnewmat{pp,yy}=(covpotnewmat{pp,yy}+covpotoldmat{pp,yy})./2;
        end
    end
    
    OP=OP-opstep;
end %Overpotential loop

%plotting the real total exchange current density in the volcano plot as a line
plot(linspace(-0.9,0.9,10),linspace(log10(i0fullsmat),log10(i0fullsmat),10),'--k')
%%
% Plotting results from input structure: Tafel plots and polarization plot
position=[600 100 350*1.2 350];
set(0, 'DefaultAxesFontSize',14,'DefaultLineLinewidth',1.0,'DefaultaxesLinewidth',1,'Defaultaxesfontweight', 'normal', 'DefaultFigurePosition', position);

figure(jd*3)
plot(OPP(:),cell2mat(IImatfull(:)).*1000,'-k') %mA/cm2
xlabel('E (V vs. SHE)')
ylabel('J (mA/cm^{2})')
ylim([-100,5])
xlim([-0.5 0])
grid on
hold on

figure(jd*4)
plot(log10(abs(cell2mat(IImatfull(:)))),(abs(OPP(:))),'k')
hold on
plot(log10(abs(cell2mat(IImatf1f(:)))),(abs(OPP(:))),'--g')
plot(log10(abs(cell2mat(IImatf1b(:)))),(abs(OPP(:))),':g')
plot(log10(abs(cell2mat(IImatf2f(:)))),(abs(OPP(:))),'--b')
plot(log10(abs(cell2mat(IImatf2b(:)))),(abs(OPP(:))),':b')
plot(log10(abs(cell2mat(IImatf3f(:)))),(abs(OPP(:))),'--r')
plot(log10(abs(cell2mat(IImatf3b(:)))),(abs(OPP(:))),':r')
plot(log10(abs(cell2mat(IImatf1t(:)))),(abs(OPP(:))),'-g')
plot(log10(abs(cell2mat(IImatf2t(:)))),(abs(OPP(:))),'-b')
plot(log10(abs(cell2mat(IImatf3t(:)))),(abs(OPP(:))),'-r')

xlim([-6 -0.5])
xlabel('log_{10}|J| (A/cm^{2})')
ylabel('\eta (V)')
grid on
ylim([0,max(abs(OPP))])

box off
hAx(1)=gca;
hAx(2)=axes('Position',hAx(1).Position,'XAxisLocation','top','YAxisLocation','left','color','none');
hold(hAx(2),'on')
yticks([''])
plot(hAx(2),cell2mat(covpotmattot(:)),(abs(OPP(:))),'--','Color',[0.85,0.33,0.1])
xlabel('Coverage (\Theta)')
ylim([0,max(abs(OPP))])
xlim([-0.05 1.05])
hold off

figure(jd*5+1)
%plot(log10(abs(cell2mat(IIPtfull(:)).*1000)),(abs(OPP(:))),'k') %mA
plot(log10(abs(cell2mat(IImatfull(:)))),(abs(OPP(:))),'k') %A
xlim([-6 -0.5])
ylim([0 max(abs(OPP))])



xlabel('log_{10}|J| (A/cm^{2})')
ylabel('\eta (V)')
grid on


%% Visualization of the final H-positions, the structure and the final GE energies
%uses x,y,z pos from final output file
filenameE=[folder 'finalpositions_run2.xyz']; %[x y z species]

fid = fopen(filenameE,'r');
xyzs = textscan(fid,'%f%f%f%f');
fclose(fid);
type=xyzs{4};

for j=1:length(type)
    if type(j)~=max(type)
        xs(j)=xyzs{1}(j);
        ys(j)=xyzs{2}(j);
        zs(j)=xyzs{3}(j);
    end
end
%For only plotting the surface atoms, use e.g 0.5 here, otherwise a larger number. 
l=1;
ztop=0.5;
for j=1:length(zs)
    if abs(zs(j)-max(zs)) < ztop
        xsurf(l)=xs(j);
        ysurf(l)=ys(j);
        zsurf(l)=zs(j);
        l=l+1;
    end
end
for j=1:length(xmat)
    if xmat(j)>xcellmat
        xmat(j)=xmat(j)-xcellmat;
    end
    if xmat(j)<1
        xmat(j)=xmat(j)+xcellmat;
    end
    if ymat(j)>ycellmat
        ymat(j)=ymat(j)-ycellmat;
    end
    if ymat(j)<1
        ymat(j)=ymat(j)+ycellmat;
    end
end

figure(jd*6)
scatter3(xsurf,ysurf,zsurf,'o','filled','SizeData',50,'MarkerFaceColor',[0.5 0.5 0.5])
hold on

line([min(xsurf),xcellmat+min(xsurf)],[min(ysurf) min(ysurf)], [min(zsurf) min(zsurf)],'Color',[0 0 0])
line([min(xsurf),min(xsurf)],[min(ysurf),ycellmat+min(ysurf)],[min(zsurf) min(zsurf)],'Color',[0 0 0])
line([xcellmat+min(xsurf),xcellmat+min(xsurf)],[min(ysurf) ycellmat+min(ysurf)],[min(zsurf) min(zsurf)],'Color',[0 0 0])
line([min(xsurf),xcellmat+min(xsurf)],[ycellmat+min(ysurf) ycellmat+min(ysurf)],[min(zsurf) min(zsurf)],'Color',[0 0 0])

scatter3(xmat,ymat,zmat,100,GEmat,'filled','o','SizeData',200);
axis equal;
box on
view(0,90)
xlim([min(min(xsurf),min(xmat))-2 max(max(xsurf),max(xmat))+2])
ylim([min(min(ysurf),min(ymat))-2 max(max(ysurf),max(ymat))+2])
set(gca,'Ytick',[0,5,10,15,20])
set(gca,'Xtick',[0,5,10,15,20])
set(gca,'YAxisLocation','right');
set(gca,'Ticklength',[0.02 0.02],'TickDir','in');

colormap(jet(5));
h = colorbar;
caxis([-0.5 0.5]);

set(h,'Ticks',[-0.5 -0.3 -0.1 0.1 0.3 0.5]);
set(h,'TickLabels',{'-0.5','-0.3', '-0.1',' 0.1',' 0.3',' 0.5'})
set(h,'TickLength',0.03)

grid on
hold off